// ===========================================================================
// 	GSColorTable.c 
// ===========================================================================


#include "GameScene_prefix.h"

#include <stdlib.h>

#if (kQAPlatform == kQAMacOS)
	#include <PictUtil.h>
#endif

#include "RAVE.h"

#include "GSColorTable.h"
#include "GSError.h"


// ===========================================================================
// 	Constants
// ===========================================================================

#define		 				kCL32bit_ElementSize 			4
#define		 				kCL8_ElementCount 				256
#define		 				kCL4_ElementCount 				16

//#if (kQAPlatform == kQAMacOS)
	#define		 				kARGB32shift_alpha				24
	#define		 				kARGB32shift_red				16
	#define		 				kARGB32shift_green				8
	#define		 				kARGB32shift_blue				0
//#elif (kQAPlatform == kQAWin32)
//	#define		 				kARGB32shift_alpha				0
//	#define		 				kARGB32shift_red				8
//	#define		 				kARGB32shift_green				16
//	#define		 				kARGB32shift_blue				24
//#endif

#define		 				kARGB32mask_alpha				0xFF << kARGB32shift_alpha
#define		 				kARGB32mask_red					0xFF << kARGB32shift_red
#define		 				kARGB32mask_green				0xFF << kARGB32shift_green
#define		 				kARGB32mask_blue				0xFF << kARGB32shift_blue


// ===========================================================================
//	Types
// ===========================================================================

typedef struct TGSColorTable {
	TQAColorTableType		mType;
	Byte*					mDataP;
} TGSColorTable;


// ===========================================================================
//	Public Routines
// ===========================================================================

// ===========================================================================
//	GSColorTable_New
// ===========================================================================
	TGSError
GSColorTable_New(
	TGSColorTable**			inColorTable)
{
	*inColorTable = (TGSColorTable*) malloc(sizeof(TGSColorTable));
	
	if (*inColorTable == nil) {
		return kGSError_NotEnoughMemory;
	}
	
	(**inColorTable).mType = kQAPixel_RGB32;
	(**inColorTable).mDataP = nil;
	
	return kGSError_None;
}


#if (kQAPlatform == kQAMacOS)

// ===========================================================================
//	GSColorTable_NewFromMacColorTable
// ===========================================================================
	TGSError
GSColorTable_NewFromMacColorTable(
	TGSColorTable**			inColorTable,
	TQAColorTableType	 	inColorTableType,
	CTabHandle				inMacColorTableH)
{
	short					elementCount = 0;
	short					i;
	TGSError				gsError;
	
	GSAssert_(inMacColorTableH);
	GSAssert_(inColorTableType >= kQACL8_RGB32 && inColorTableType <= kQACL4_RGB32);
	
	gsError = GSColorTable_New(inColorTable);
	
	if (gsError != kGSError_None) {
		GSColorTable_Delete(*inColorTable);
		return gsError;
	}
	
	(**inColorTable).mType = inColorTableType;
	
	switch ((**inColorTable).mType) {
		case kQACL8_RGB32:
			elementCount = kCL8_ElementCount;
			break;
		
		case kQACL4_RGB32:
			elementCount = kCL4_ElementCount;
			break;
		
		default:
			GSAssert_(!"Found unrecognized TQAColorTableType in GSColorTable_NewFromMacColorTable\n");
			break;
	} // endswitch
	
	(**inColorTable).mDataP = malloc(elementCount * kCL32bit_ElementSize);
	
	if ((**inColorTable).mDataP == nil) {
		GSColorTable_Delete(*inColorTable);
		return kGSError_NotEnoughMemory;
	}

	for (i = 0; i < elementCount; i++) {
		unsigned long*		elementP = (unsigned long*) (**inColorTable).mDataP;
		unsigned long 		channelValue;
		
			// turn on all the bits in the alpha channel.  we do this
			// for every entry in the TQAColorTable, regardless of how
			// many entries are in inMacColorTableH.  this also sets
			// the color to black.
		elementP[i] = kARGB32mask_alpha;
		
		if (i <= (**inMacColorTableH).ctSize) {
				// we haven't run out of ColorTable entries yet, so reduce
				// each 16-bit channel in the entry to 8 bits and incorporate
				// it into the TQAColorTable entry.
			channelValue = (**inMacColorTableH).ctTable[i].rgb.red >> 8;
			elementP[i] |= ((channelValue << kARGB32shift_red) & kARGB32mask_red);

			channelValue = (**inMacColorTableH).ctTable[i].rgb.green >> 8;
			elementP[i] |= ((channelValue << kARGB32shift_green) & kARGB32mask_green);

			channelValue = (**inMacColorTableH).ctTable[i].rgb.blue >> 8;
			elementP[i] |= ((channelValue << kARGB32shift_blue) & kARGB32mask_blue);
		}
	} // endfor
	
	return kGSError_None;
}

#endif /* kQAPlatform == kQAMacOS */

#if (kQAPlatform == kQAWin32)

TGSError	GSColorTable_NewFromWin32ColorTable(
	TGSColorTable**			inColorTable,
	TQAColorTableType	 	inColorTableType,
	int						inColorCount,
	RGBQUAD					*inWin32Colors)								
{
	short					elementCount = 0;
	short					i;
	TGSError				gsError;

	GSAssert_(inWin32Colors);
	GSAssert_(inColorTableType >= kQAColorTable_CL8_RGB32 && inColorTableType <= kQAColorTable_CL4_RGB32);

	gsError = GSColorTable_New(inColorTable);
	if (gsError != kGSError_None)
		goto ExitGSColorTable_NewFromWin32ColorTable;

	(**inColorTable).mType = inColorTableType;
	switch ((**inColorTable).mType)
	{
		case kQAColorTable_CL8_RGB32:
			elementCount = kCL8_ElementCount;
			break;
		
		case kQAColorTable_CL4_RGB32:
			elementCount = kCL4_ElementCount;
			break;
		
		default:
			GSAssert_(!"Found unrecognized TQAColorTableType in GSColorTable_NewFromMacColorTable");
			break;
	}
	(**inColorTable).mDataP = malloc(elementCount * kCL32bit_ElementSize);
	if ((**inColorTable).mDataP == nil)
	{
		gsError = kGSError_NotEnoughMemory;
		goto ExitGSColorTable_NewFromWin32ColorTable;
	}
	for (i = 0; i < elementCount; i++)
	{
		unsigned long*		elementP = (unsigned long*) (**inColorTable).mDataP;
		unsigned long 		channelValue;

			// Turn on all the bits in the alpha channel.  Do this
			// for every entry in the TQAColorTable, regardless of how
			// many entries are in inMacColorTableH.  This also sets
			// the color to black.
		elementP[i] = (unsigned long)kARGB32mask_alpha;
		if (i <= inColorCount)
		{
				// Haven't run out of entries yet, so incorporate
				// it into the TQAColorTable entry.
			channelValue = inWin32Colors[i].rgbRed;
			elementP[i] |= ((channelValue << kARGB32shift_red) & kARGB32mask_red);

			channelValue = inWin32Colors[i].rgbGreen;
			elementP[i] |= ((channelValue << kARGB32shift_green) & kARGB32mask_green);

			channelValue = inWin32Colors[i].rgbBlue;
			elementP[i] |= ((channelValue << kARGB32shift_blue) & kARGB32mask_blue);
		}
	/*	elementP[i] = kARGB32mask_alpha | (255L << kARGB32shift_blue); */
	}
ExitGSColorTable_NewFromWin32ColorTable:
	if (gsError != kGSError_None)
		GSColorTable_Delete(*inColorTable);

	return gsError;
}

#endif /* kQAPlatform == kQAWin32 */

// ===========================================================================
//	GSColorTable_Delete
// ===========================================================================
	void
GSColorTable_Delete(
	TGSColorTable*			inColorTable)
{
	if (inColorTable == nil) {
		return;
	}
	
	free(inColorTable);
}


// ===========================================================================
//	GSColorTable_ChangeBrightness
// ===========================================================================
	TGSError
GSColorTable_ChangeBrightness(
	TGSColorTable*			inColorTable,
	float					inBrightnessPercentage)
{
	short					elementCount = 0;
	short					i;

	GSAssert_(inColorTable);
	GSAssert_(inColorTable->mDataP != nil);
	
	switch (inColorTable->mType) {
		case kQAColorTable_CL8_RGB32:
			elementCount = kCL8_ElementCount;
			break;
		
		case kQAColorTable_CL4_RGB32:
			elementCount = kCL4_ElementCount;
			break;
		
		default:
			GSAssert_(!"Found unrecognized TQAColorTableType in GSColorTable_NewFromMacColorTable");
			break;
	} // endswitch
	
	for (i = 0; i < elementCount; i++) {
		Byte*				channelP = (Byte*) inColorTable->mDataP;
		unsigned long		channelValue;
		
			// skip past the alpha channel, which is the first one
		channelP++;
		
		channelValue = (unsigned long) (*channelP * inBrightnessPercentage);
		*channelP = (Byte) (channelValue > 0xFF) ? 0xFF : channelValue;
		channelP++;

		channelValue = (unsigned long) (*channelP * inBrightnessPercentage);
		*channelP = (Byte) (channelValue > 0xFF) ? 0xFF : channelValue;
		channelP++;

		channelValue = (unsigned long) (*channelP * inBrightnessPercentage);
		*channelP = (Byte) (channelValue > 0xFF) ? 0xFF : channelValue;
		channelP++;
	} // endfor
	
	return kGSError_None;
}


// ===========================================================================
//	GSColorTable_GetData
// ===========================================================================
	void*
GSColorTable_GetData(
	TGSColorTable*			inColorTable)
{
	GSAssert_(inColorTable);
	
	return inColorTable->mDataP;
}


// ===========================================================================
//	GSColorTable_GetType
// ===========================================================================
	TQAColorTableType
GSColorTable_GetType(
	TGSColorTable*			inColorTable)
{
	GSAssert_(inColorTable);
	
	return inColorTable->mType;
}


