/******************************************************************************
 **																			 **
 ** 	Module:		QAL_Window.c											 **
 ** 																		 **
 ** 	Purpose: 	Window/GDevice stuff for QAL test app					 **
 ** 																		 **
 ** 	Author:		Mike W. Kelley											 **
 **																			 **
 **					2/3/95	Revised for 0.9 SDK release						 **
 ** 																		 **
 ** 	Copyright (C) 1994-95 Apple Computer, Inc.  All rights reserved.	 **
 ** 																		 **
 *****************************************************************************/

#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <stdio.h>

#include "RAVE.h"

#if (kQAPlatform == kQAMacOS)
	/* Macintosh */
	#include <Types.h>
	#include <Resources.h>
	#include <QuickDraw.h>
	#include <Fonts.h>
	#include <Events.h>
	#include <Windows.h>
	#include <Menus.h>
	#include <TextEdit.h>
	#include <Dialogs.h>
	#include <Desk.h>
	#include <ToolUtils.h>
	#include <Memory.h>
	#include <SegLoad.h>
	#include <Files.h>
	#include <OSUtils.h>
	#include <OSEvents.h>
	#include <DiskInit.h>
	#include <Packages.h>
#endif /* kQAMacOS */

	/* Choose which type of device you want to test by defining to 1 */
#define kTestWin32DC	1
#define kTestDDSurface	0

#if (kQAPlatform == kQAWin32)
	#include <windows.h>
	#if defined(kTestDDSurface) && kTestDDSurface
		#include <ddraw.h>
	#endif
#endif

/* Project */
#include "QAL_Test.h"

/*******************************************************************************************
 *
 * Static variables used by this file's functions only.
 *
 ******************************************************************************************/

#if (kQAPlatform == kQAMacOS)
static	GDHandle		gGDevice;
static	WindowPtr		gQAL_Window [kQAL_Windows];
static	RgnHandle		gClipRgn [kQAL_Windows];
#endif /* kQAMacOS */

#if (kQAPlatform == kQAWin32)
static HWND				gQAL_Window [kQAL_Windows];
static HRGN				gClipRgn [kQAL_Windows];

char szTitle[]   = "RAVE Demo";		// The title bar text

extern char		szAppName[];

#if defined(kTestDDSurface) && kTestDDSurface
	LPDIRECTDRAWSURFACE	gPrimarySurface = NULL;
	LPDIRECTDRAW		gDirectDraw = NULL;
#endif

#endif /* kQAWin32 */

/*******************************************************************************************
 *
 * Utility functions
 *
 ******************************************************************************************/

#if (kQAPlatform == kQAMacOS)
void InitToolbox (void)
{
	InitGraf((Ptr) &qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	FlushEvents(everyEvent,0);
	TEInit();
	InitDialogs(0L);
	InitCursor();
}
#endif /* kQAMacOS */

/************************************************************************************************
 *
 * Error handling stuff.
 *
 ***********************************************************************************************/

void QAL_Error (
	const char		*message)
{
	char			ErrorStr[256];

	sprintf (&ErrorStr[1], "QAL_Error: %s\n", message);
#if (kQAPlatform == kQAMacOS)
	ErrorStr[0] = strlen (&ErrorStr[1]) - 1;
	DebugStr ( (ConstStr255Param) ErrorStr);
#endif
}

/************************************************************************************************
 *
 * Choose a drawing engine for 'windowNumber'. This returns the engine name.
 *
 ***********************************************************************************************/

static TQAEngine *ChooseEngine (
	TQADevice		*device,
	long			windowNumber,
	char			engineName[64])		/* Out PString */
{
	TQAEngine		*engine;
	long			i;
	long			alreadyUsed;
	TQAError		gestaltStatus;
	long			engineNameLength;
	
	for (engine = QADeviceGetFirstEngine (device);
			engine;
			engine = QADeviceGetNextEngine (device, engine))
	{
		/*
		 * Search to see if this engine is already being used.
		 */
		for (i = 0, alreadyUsed = FALSE; i < windowNumber; ++i)
		{
			if (gEngine [i] == engine)
			{
				alreadyUsed = TRUE;
			}
		}
		
		if ( ! alreadyUsed)
		{
			/*
			 * Use this engine.
			 */
			break;
		}
	}

	if (engine)
	{
		/*
		 * A unique drawing engine was found. Do a gestalt call to get the
		 * engine name.
		 */
		gestaltStatus = QAEngineGestalt (engine, kQAGestalt_ASCIINameLength, &engineNameLength);
		if ((gestaltStatus == kQANoErr) && (engineNameLength < 63))
		{
			gestaltStatus = QAEngineGestalt (engine, kQAGestalt_ASCIIName, &engineName[1]);
		}
		
		if (gestaltStatus != kQANoErr)
		{
			/*
			 * Gestalt call failed!
			 */
			
			sprintf (&engineName[1], "<Gestalt Error>");
		}
	}
	else
	{
		/*
		 * No unused engine was found. Assign a NULL engine pointer, and
		 * set a dummy engine name.
		 */
		sprintf (&engineName[1], "<No Drawing Engine>");
	}
	engineName[0] = strlen (&engineName[1]);
	
	return (engine);
}

/*******************************************************************************************
 *
 * Create test windows.
 *
 ******************************************************************************************/
#if (kQAPlatform == kQAMacOS)
	long CreateTestWindows (
		long			maxWidth,
		long			maxHeight)
#elif (kQAPlatform == kQAWin32)
	long CreateTestWindows(
		long			maxWidth,
		long			maxHeight,
        HINSTANCE       hInstance,
        int             nCmdShow)
#endif
{
	long			windowWidth, windowHeight;
	TQADevice		device;
	char			engineName[64];
#if (kQAPlatform == kQAMacOS)
	GDHandle		colourGDevice, deepestGDevice;
	long			deviceDepth, maxDepth;
	Rect			deviceRect;
	Rect			windowRect;
	long			deviceWidth, deviceHeight;

	/*
	 * Find the deepest color display.
	 */
	maxDepth = 0;
	for (colourGDevice = GetDeviceList(), deepestGDevice = NULL;
			colourGDevice;
			colourGDevice = GetNextDevice (colourGDevice))
	{
		deviceDepth = (*(*colourGDevice)->gdPMap)->pixelSize;
		if (deviceDepth > maxDepth)
		{
			maxDepth = deviceDepth;
			deepestGDevice = colourGDevice;
		}
	}
	gGDevice = deepestGDevice;
	/*
	 * Open two side-by-side windows which almost fill the device, or to
	 * maxWidth, maxHeight.
	 */
	deviceRect = (*deepestGDevice)->gdRect;
	
	deviceWidth = (deviceRect.right - deviceRect.left) - 2 * kDeviceWindowBorder;
	deviceHeight = (deviceRect.bottom - deviceRect.top) - 2 * kDeviceWindowBorder;
	
	windowWidth = (deviceWidth / 2) - 2 * kDeviceWindowBorder;
	windowHeight = deviceHeight - 2 * kDeviceWindowBorder;
	
	if (windowWidth > maxWidth)
		windowWidth = maxWidth;
	if (windowHeight > maxHeight)
		windowHeight = maxHeight;
	
	gWindowWidth = windowWidth;
	gWindowHeight = windowHeight;
	
	device.deviceType = kQADeviceGDevice;
	device.device.gDevice = deepestGDevice;

	/*
	 * Choose a drawing engine for the left window, and create the window.
	 */
	gEngine[0] = ChooseEngine (&device, 0, engineName);
	
	windowRect.top = (deviceRect.top + deviceRect.bottom) / 2 - windowHeight / 2;
	windowRect.bottom = windowRect.top + windowHeight;

	windowRect.left = (deviceRect.left * 3 + deviceRect.right) / 4 - windowWidth / 2;
	windowRect.right = windowRect.left + windowWidth;

	gQAL_Window[0] = NewCWindow (nil, &windowRect, (ConstStr255Param) engineName, true, zoomDocProc, nil, true, 0);
	
	/*
	 * Choose a drawing engine for the right window, and create the window.
	 */
	gEngine[1] = ChooseEngine (&device, 1, engineName);

	windowRect.left = (deviceRect.left + deviceRect.right * 3) / 4 - windowWidth / 2;
	windowRect.right = windowRect.left + windowWidth;

	gQAL_Window[1] = NewCWindow (nil, &windowRect, (ConstStr255Param) engineName, true, zoomDocProc, nil, true, 0);

#elif (kQAPlatform == kQAWin32) 
	RECT			deviceRect, clientRect;
	int				winX, winY, maxDisplayX, maxDisplayY;
	BOOL			aSetResult;

	maxDisplayX = GetSystemMetrics(SM_CXFULLSCREEN); /* get display size */
	maxDisplayY = GetSystemMetrics(SM_CYFULLSCREEN);
  
	deviceRect.top = 0;
	deviceRect.left = 0;
	deviceRect.right = maxDisplayX;
	deviceRect.bottom = maxDisplayY;

	windowWidth = ((deviceRect.right - deviceRect.left) / 2) - (2 * kDeviceWindowBorder);
	windowHeight = ((deviceRect.bottom - deviceRect.top) / 2) - (2 * kDeviceWindowBorder);

	if (windowWidth > maxWidth)
		windowWidth = maxWidth;
	if (windowHeight > maxHeight)
		windowHeight = maxHeight;

	winX = deviceRect.left + kDeviceWindowBorder;
	winY = deviceRect.top + kDeviceWindowBorder;
	gQAL_Window[0] = CreateWindow(
            szAppName,          
            &engineName[0],             
            WS_OVERLAPPEDWINDOW, 
            winX, winY, windowWidth, windowHeight, 	 // x, y, width, height
            NULL,                
            NULL,                
            hInstance,           
            NULL                 
	);
 	winX = deviceRect.left + kDeviceWindowBorder + windowWidth + kDeviceWindowBorder;
	winY = deviceRect.top + kDeviceWindowBorder;
	gQAL_Window[1] = CreateWindow(
            szAppName,          
            &engineName[1],             
            WS_OVERLAPPEDWINDOW, 
            winX, winY, windowWidth, windowHeight, 	 // x, y, width, height
            NULL,                
            NULL,                
            hInstance,           
            NULL                 
    );
	GetClientRect(gQAL_Window[1], (LPRECT)&clientRect);
	gWindowWidth = clientRect.right - clientRect.left;
	gWindowHeight = clientRect.bottom - clientRect.top;

	if (gQAL_Window[0] && gQAL_Window[1]) 
	{
		ShowWindow(gQAL_Window[0], nCmdShow); 
	/*	UpdateWindow(gQAL_Window[0]);	*/		/* send WM_PAINT to window */

		ShowWindow(gQAL_Window[1], nCmdShow); 
	/*	UpdateWindow(gQAL_Window[1]); */

#if defined(kTestWin32DC) && kTestWin32DC
		/* Choose a drawing engine for the left window, and create the window. */
		device.deviceType = kQADeviceWin32DC;
		device.device.hdc = GetDC(gQAL_Window[0]);
		gEngine[0] = ChooseEngine (&device, 0, engineName);
		aSetResult = SetWindowText(gQAL_Window[0], (LPCTSTR)&engineName[1]);

		device.device.hdc = GetDC(gQAL_Window[1]);
		gEngine[1] = ChooseEngine (&device, 1, engineName);
		aSetResult = SetWindowText(gQAL_Window[1], (LPCTSTR)&engineName[1]);
#elif defined(kTestDDSurface) && kTestDDSurface
		{
			HRESULT             ddrval;
			DDSURFACEDESC		ddsd;

			device.deviceType = kQADeviceDDSurface;
			ddrval = DirectDrawCreate( NULL, &device.device.lpDirectDraw, NULL );
			gDirectDraw = device.device.lpDirectDraw;

            ddrval = IDirectDraw_SetCooperativeLevel( gDirectDraw, gQAL_Window[0],
                            DDSCL_NORMAL );

			ddsd.dwSize = sizeof(DDSURFACEDESC);
			ddsd.dwFlags = DDSD_CAPS;
			ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
			ddrval = IDirectDraw_CreateSurface(
				device.device.lpDirectDraw,
				&ddsd,
				&device.device.lpDirectDrawSurface, NULL);
			gPrimarySurface = device.device.lpDirectDrawSurface;

			gEngine[0] = ChooseEngine (&device, 0, engineName);
			aSetResult = SetWindowText(gQAL_Window[0], (LPCTSTR)&engineName[1]);

			gEngine[1] = ChooseEngine (&device, 1, engineName);
			aSetResult = SetWindowText(gQAL_Window[1], (LPCTSTR)&engineName[1]);
		}
#endif /* kTestWin32DC else kTestDDSurface */
	}

#endif /* kQAWin32 */

	/* If both window creations were successful, return true. */
	return (gQAL_Window[0] && gQAL_Window[1]);
}

/*******************************************************************************************
 *
 * Create test windows on the deepest GDevice.
 *
 ******************************************************************************************/

void ClearWindows (void)
{
	long			i;
#if (kQAPlatform == kQAMacOS)
	WindowPtr		window;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		GrafPtr		previousPort;
		
		window = gQAL_Window [i];
		
		GetPort (&previousPort);
		SetPort (window);
		EraseRect (&window->portRect);
		SetPort (previousPort);
	}
#elif (kQAPlatform == kQAWin32)
	HWND		hWnd;
	HDC			hdc;
	RECT		aWinRect;
	BOOL		aResult;				

	for (i = 0; i < kQAL_Windows; ++i)
	{
		hWnd = gQAL_Window [i];
		hdc = GetDC(hWnd);
		aResult = GetClientRect(hWnd, (LPRECT)&aWinRect);				
		FillRect(hdc, &aWinRect, (HBRUSH) (COLOR_WINDOW+1));				
	}
#endif
}

/*******************************************************************************************
 *
 * Create test clip region (a round rect) for each test window.
 *
 ******************************************************************************************/

long CreateWindowClip (void)
{
	long			i;

#if (kQAPlatform == kQAMacOS)
	Point 			topLeft, botRight;
	WindowPtr		window;
	Rect			windowRect, clipRect;

	for (i = 0; i < kQAL_Windows; ++i)
	{
		GrafPtr		previousPort;

		window = gQAL_Window [i];
		
		/* Get the boundaries of the target window, and compute its size. */
		topLeft.v = window->portRect.top;
		topLeft.h = window->portRect.left;
		botRight.v = window->portRect.bottom;
		botRight.h = window->portRect.right;
	
		/* Convert window boundaries to global co-ordinates. */
		GetPort (&previousPort);
		SetPort (window);
		LocalToGlobal (&topLeft);
		LocalToGlobal (&botRight);
		SetPort (previousPort);
	
		/* Create the window bound rectangle in device coordinates. */
		windowRect.top = topLeft.v - (*gGDevice)->gdRect.top;
		windowRect.left = topLeft.h - (*gGDevice)->gdRect.left;
		windowRect.bottom = botRight.v - (*gGDevice)->gdRect.top;
		windowRect.right = botRight.h - (*gGDevice)->gdRect.left;
		
		/* Build the clip region for this window. */
		if ( ! (gClipRgn[i] = NewRgn()))
		{
			return (false);
		}
		
		OpenRgn();
		
		clipRect.top = windowRect.top + kWindowClipBoundary;
		clipRect.left = windowRect.left + kWindowClipBoundary;
		clipRect.bottom = windowRect.bottom - kWindowClipBoundary;
		clipRect.right = windowRect.right - kWindowClipBoundary;
		
		FrameRoundRect (&clipRect, kClipRoundRect, kClipRoundRect);
		CloseRgn (gClipRgn[i]);
	}
#endif /* kQAMacOS */

#if (kQAPlatform == kQAWin32)
	HWND	aWindow;
	RECT	aRect, aClipRect;
	BOOL	aResult;

	for (i = 0; i < kQAL_Windows; ++i)
	{
		aWindow = gQAL_Window[i];
		aResult = GetClientRect(aWindow, (LPRECT)&aRect);
		aClipRect.top = aRect.top + kWindowClipBoundary;
		aClipRect.left = aRect.left + kWindowClipBoundary;
		aClipRect.bottom = aRect.bottom - kWindowClipBoundary;
		aClipRect.right = aRect.right - kWindowClipBoundary;
		gClipRgn[i] = CreateRoundRectRgn(
			aClipRect.left, aClipRect.top,
			aClipRect.right, aClipRect.bottom,
			kClipRoundRect, kClipRoundRect);
	}
#endif /* kQAWin32 */

	return (TRUE);
}

/*******************************************************************************************
 *
 * Create draw contexts for the test windows.
 *
 ******************************************************************************************/

long TestDrawContextNew (
	unsigned long	flags,
	long			do2DClipping)
{
	TQARect				windowRect;
	long				i;
	TQADevice			device;
	TQAError			status;
	TQAClip				clip;
#if (kQAPlatform == kQAMacOS)
	WindowPtr			window;
	Point 				topLeft, botRight;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gEngine [i])
		{
			
			window = gQAL_Window [i];
			
			/* Get the boundaries of the target window, and compute its size. */
			topLeft.v = window->portRect.top;
			topLeft.h = window->portRect.left;
			botRight.v = window->portRect.bottom;
			botRight.h = window->portRect.right;
		
			/* Convert window boundaries to global co-ordinates. */
			SetPort (window);
			LocalToGlobal (&topLeft);
			LocalToGlobal (&botRight);
		
			/* Create the bound rectangle in device coordinates. */
			windowRect.top = topLeft.v - (*gGDevice)->gdRect.top;
			windowRect.left = topLeft.h - (*gGDevice)->gdRect.left;
			windowRect.bottom = botRight.v - (*gGDevice)->gdRect.top;
			windowRect.right = botRight.h - (*gGDevice)->gdRect.left;
		
			/* Create the draw context. */
			device.deviceType = kQADeviceGDevice;
			device.device.gDevice = gGDevice;
	
			clip.clipType = kQAClipRgn;
			clip.clip.clipRgn = gClipRgn[i];
			
			status = QADrawContextNew (&device, &windowRect,
					do2DClipping ? &clip : NULL, gEngine[i], flags, &gDrawContext[i]);
			
			if (status != kQANoErr)
			{
				QAL_Error ("Could not create draw context");
				return (FALSE);
			}
		}
		else
			gDrawContext [i] = NULL;   /* No drawing engine attached to this window. */
	}
#elif (kQAPlatform == kQAWin32)
	HWND		window;
	RECT		aWinRect;
	BOOL		aResult;
#if defined(kTestDDSurface) && kTestDDSurface
	POINT		point;
#endif

	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gEngine [i])
		{
			window = gQAL_Window [i];
			aResult = GetClientRect(window, (LPRECT)&aWinRect);
			windowRect.top = aWinRect.top;
			windowRect.left = aWinRect.left;
			windowRect.bottom = aWinRect.bottom;
			windowRect.right = aWinRect.right;
#if defined(kTestWin32DC) && kTestWin32DC
			device.deviceType = kQADeviceWin32DC;
			device.device.hdc = GetDC(window);
#elif defined(kTestDDSurface) && kTestDDSurface
			point.x = windowRect.left;
			point.y = windowRect.top;
			ClientToScreen(window, &point);
			windowRect.left += point.x;
			windowRect.right += point.x;
			windowRect.top += point.y;
			windowRect.bottom += point.y;
			device.deviceType = kQADeviceDDSurface;
			device.device.lpDirectDraw = gDirectDraw;
			device.device.lpDirectDrawSurface = gPrimarySurface;
#endif
			clip.clipType = kQAClipWin32Rgn;
			clip.clip.clipRgn = gClipRgn[i];
			status = QADrawContextNew (&device, &windowRect,
					do2DClipping ? &clip : NULL, gEngine [i], flags, &gDrawContext[i]);
			if (status != kQANoErr)
			{
				QAL_Error ("Could not create draw context");
				return (FALSE);
			}
		}
		else
			gDrawContext [i] = NULL;   /* No drawing engine attached to this window. */
	}
#endif /* kQAWin32 */
	return (TRUE);
}

void TestDrawContextDelete (void)
{
	long		i;
	
	for (i = 0; i < kQAL_Windows; ++i)
	{
		if (gDrawContext [i])
		{
			QADrawContextDelete (gDrawContext [i]);
			gDrawContext [i] = NULL;
		}
	}
}


